// Es.5.1
#include <stdio.h> // Per printf() e scanf()
#include <stdlib.h> // Per rand()

#define MAX_LENGTH 4
#define MAX_ROUNDS 10

// Costanti colori
#define RED 1
#define YELLOW 2
#define GREEN 3
#define ORANGE 4
#define BLACK 5
#define WHITE 6

struct settings{
	int code_length;
	int max_rounds;
} impostazioni;


// Genera casualmente la sequenza segreta
// secret_code: vettore sequenza CPU
void genera_codice(char* secret_code){
	int i, randColor;
	/**(secret_code) = 'w';
	*(secret_code+1) = 'o';
	*(secret_code+2) = 'b';
	*(secret_code+3) = 'r';*/

	for(i=0; i<impostazioni.code_length; i++){

		randColor = 1+rand()%6;
		switch(randColor){
			case RED:
				*(secret_code+i) = 'r';
				break;
			case YELLOW:
				*(secret_code+i) = 'y';
				break;
			case GREEN:
				*(secret_code+i) = 'g';
				break;
			case ORANGE:
				*(secret_code+i) = 'o';
				break;
			case BLACK:
				*(secret_code+i) = 'b';
				break;
			case WHITE:
				*(secret_code+i) = 'w';
				break;
		}
	}

}

// Permette l'inserimento della sequenza da parte dell'utente
// guess_code: vettore sequenza utente
void indovina_codice(char* guess_code){
	int i;
	printf("Inserisci la possibile soluzione (lunghezza %d):\n", impostazioni.code_length);
	for(i=0; i<impostazioni.code_length; i++){
		scanf(" %c", (guess_code+i));
	}
}

// Logica di gioco che controlla se la sequenza
// inserita è corretta e ritorna il risultato del round
// secret_code e guess_code: vettori alle sequenze CPU e utente
// white_pegs: puntatore a intero numero di pallini bianchi a fine round
// black_pegs: puntatore a intero numero di pallini neri a fine round
// Ritorna 1 se l'utente ha vinto, 0 altrimenti
int controlla_codice(char* secret_code, char* guess_code, int* white_pegs, int* black_pegs){
	int i,j;
	int check_secret[MAX_LENGTH] = {1,1,1,1};
	int check_guess[MAX_LENGTH] = {1,1,1,1};

	// Resetto contatori
	*white_pegs = *black_pegs = 0;

	// Controllo colore e posizione
	for(i=0; i<impostazioni.code_length; i++){
		if(*(secret_code+i) == *(guess_code+i)){
			*black_pegs = *black_pegs + 1;
			check_secret[i] = 0;
			check_guess[i] = 0;
		}
	}

	for(i=0; i<impostazioni.code_length; i++){
		for(j=0; j<impostazioni.code_length; j++){
			if((*(secret_code+i) == *(guess_code+j)) && check_guess[j] && check_secret[i] && i!=j){
				*white_pegs = *white_pegs + 1;
				check_secret[i] = 0;
				check_guess[j] = 0;
			}
		}
	}

	if(*black_pegs == impostazioni.code_length)
		return 1;

	return 0;
}

// Stampa la sequenza inserita dall'utente
// code: vettore sequenza utente
void stampa_codice(char* code){
	int i;

	for(i=0; i<impostazioni.code_length; i++){
		printf("%c ", *(code+i));
	}

	printf("\t|\t");
}

// Stampa il risultato alla fine del round
// white_pegs: numero di pallini bianchi
// black_pegs: numero di pallini neri
void stampa_risultato(int white_pegs, int black_pegs){
	int i;

	for(i=0; i<black_pegs; i++)
		printf("b ");
	for(i=0; i<white_pegs; i++)
		printf("w ");
	printf("\n");
	
}

// Stampa la vittoria utente o CPU più il codice segreto
// vittoria: variabile che indica se l'utente ha vinto
// secret_code: vettore sequenza CPU
void stampa_vittoria(int vittoria, char* secret_code){
	printf("La soluzione era: %s\n", secret_code);
	if(vittoria == 1){
		printf("Congratulazioni hai vinto!\n");
		return;
	}
	printf("Hai perso!\n");
}

/*--------------------------------------------------*/
/*                  Nuove funzioni                  */
/*--------------------------------------------------*/

// Salva il corrente round su file
// fp: file pointer
// partita: puntatore alla corrente partita
// Ritorna 0 se OK, <>0 altrimenti
int salva_round(FILE* fp, 
	char* secret_code, 
	char* guess_code, 
	int white_pegs, 
	int black_pegs, 
	int current_round){
	fp = fopen("round.txt", "w");
	
	// Verifico se c'è stato un errore nell'apertura del file
	if(fp == NULL){
		printf("Errore: Impossibile salvare il file\n");
		return -1;
	}

	// Salvo su file
	fwrite(&impostazioni.code_length, sizeof(int), 1, fp); 			// code_length
	fwrite(&impostazioni.max_rounds, sizeof(int), 1, fp); 			// max_rounds
	fwrite(&current_round, sizeof(int), 1, fp);						// current_round
	fwrite(secret_code, sizeof(char), impostazioni.code_length, fp);// secret_code
	fwrite(guess_code, sizeof(char), impostazioni.code_length, fp);	// guess_code
	fwrite(&white_pegs, sizeof(int), 1, fp);						// white_pegs
	fwrite(&black_pegs, sizeof(int), 1, fp);						// black_pegs

	return fclose(fp);
}

// Carica un precedente round da file
// fp: file pointer
// partita: puntatore alla corrente partita
// Ritorna 1 se una precedente partita esiste, 0 altrimenti
int carica_round(FILE* fp, 
	char* secret_code, 
	char* guess_code, 
	int* white_pegs, 
	int* black_pegs, 
	int* current_round){
	
	int lettura;

	fp = fopen("round.txt", "r");


	// Verifico se c'è stato un errore nell'apertura del file
	if(fp == NULL){
		printf("Errore: Impossibile aprire il file\n");
		return -1;
	}

	// Leggo da file
	lettura = fread(&impostazioni.code_length, sizeof(int), 1, fp); 	// code_length
	if(lettura>0){
		fread(&impostazioni.max_rounds, sizeof(int), 1, fp); 			// max_rounds
		fread(current_round, sizeof(int), 1, fp);						// current_round
		fread(secret_code, sizeof(char), impostazioni.code_length, fp);	// secret_code
		fread(guess_code, sizeof(char), impostazioni.code_length, fp);	// guess_code
		fread(white_pegs, sizeof(int), 1, fp);							// white_pegs
		fread(black_pegs, sizeof(int), 1, fp);							// black_pegs
		lettura = 1;
	}else{
		printf("Nessun round salvato...\n");
		lettura = 0;
	}

	fclose(fp);
	return lettura;
}

// Azzera il file precedentemente salvato
// fp: file pointer
// Ritorna 0 se OK, <>0 altrimenti
int azzera_file(FILE * fp){
	fp = fopen("round.txt", "w");
	
	// Verifico se c'è stato un errore nell'apertura del file
	if(fp == NULL){
		printf("Errore: Impossibile azzerare il file\n");
		return -1;
	}

	return fclose(fp);
}

int main(int argc, char* argv[]) {

	// Variabili di gioco
	int current_round = 0;
	char secret_code[MAX_LENGTH];
	char guess_code[MAX_LENGTH];
	int white_pegs = 0, black_pegs = 0;
	int vittoria = 0;

	int i;
	int carica_file = 0;
	char risposta_utente;

	// Variabile per file
	FILE *fp;

	// Inizializzo le impostazioni
	impostazioni.code_length = MAX_LENGTH;
	impostazioni.max_rounds = MAX_ROUNDS;

	// Verifico se l'utente vuole riprendere una partita
	printf("Vuoi riprendere una partita precedente (se esiste?)[y/N]: ");
	scanf("%c", &risposta_utente);
	switch(risposta_utente){
		case 'y':
			carica_file = carica_round(fp, secret_code, guess_code, &white_pegs, &black_pegs, &current_round);
			stampa_codice(guess_code);
			stampa_risultato(white_pegs, black_pegs);
			break;
		default:
			if(azzera_file(fp) != 0)
				return -1;
			break;
	}

	if(carica_file == 0){
		// Resetto variabili di gioco
		current_round = 0;
		white_pegs = black_pegs = 0;

		// Recupero le impostazioni passate dall'utente
		if(argc > 1){
			impostazioni.code_length = atoi(argv[1]);
			if(argc > 2)
				impostazioni.max_rounds = atoi(argv[2]);
		}

		// Genero il codice segreto di CPU
		genera_codice(secret_code);
	}
	

	while(current_round < impostazioni.max_rounds){
		printf("Round %d: ", current_round+1);
		// Faccio inserire all'utente una possibile sequenza
		indovina_codice(guess_code);
		// Stampo il codice inserito dall'utente
		stampa_codice(guess_code);
		// Controllo il codice inserito
		vittoria = controlla_codice(secret_code, guess_code, &white_pegs, &black_pegs);
		// Stampo il risultato del round
		stampa_risultato(white_pegs, black_pegs);

		if(vittoria == 1)
			break;

		current_round++;
			
		if(salva_round(fp, secret_code, guess_code, white_pegs, black_pegs, current_round) != 0)
			return -1;
		
	}

	stampa_vittoria(vittoria, secret_code);
	if(azzera_file(fp) != 0)
		return -1;

	return 0;
}
