// Es 5.4
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAGIC_NUMBER_LEN 2
#define LARG 24
#define ALTEZ 24
#define MAXLINE 1024

struct image {
	char tipo[MAGIC_NUMBER_LEN];
	int larghezza;
	int altezza;
	int pixel[ALTEZ][LARG];
};

// Funzione che legge l'hedaer del file PBM
// fp: puntatore al file
// immagine: puntatore all'immagine
// Ritorna 0 se OK, -1 se file non supportato, 1 se file troppo grande
int leggi_header(FILE *fp, struct image *immagine){
	int flag=0;
  	int larghezza, altezza;
  	unsigned int i;
  	char magic[MAXLINE];
  	char line[MAXLINE];
  	int count=0;

	while (fgets(line, MAXLINE, fp) != NULL) {
		flag = 0;

		// Rilevo i commenti
	    for (i = 0; i < strlen(line); i++) {
	        if ((line[i] == '#') && (flag == 0)) {
	          flag = 1;
	        }
	    }

	    // Se non è un commento recupero tipo e dimensioni immagine
	    if (flag == 0) {
	      if (count == 0) {
	        count += sscanf(line, "%s %d %d", magic, &larghezza, &altezza);
	      } else if (count == 1) {
	        count += sscanf(line, "%d %d", &larghezza, &altezza);
	      } else if (count == 2) {
	        count += sscanf(line, "%d", &altezza);
	      }
	    }
	    if (count == 3) {
	      break;
	    }
	}

	if (strcmp(magic, "P1") != 0) {
	    printf("Errore: Input file non in formato PBM!\n");
		return -1;
	}

	strcpy(immagine->tipo, magic);
	immagine->larghezza = larghezza;
	immagine->altezza = altezza;

	if(larghezza > LARG || altezza > ALTEZ){
		printf("Errore: Immagine troppo grande\n");
		return -1;
	}

	return 0;
}

// Funzione che legge i dati dei pixel del file PBM
// fp: puntatore al file
// immagine: puntatore all'immagine
void leggi_dati(FILE *fp, struct image *immagine){
	int i = 0, j = 0, c;
  	int pixel_val;
  	int k;
  
  	// Leggo il resto del file PBM
  	while (!feof(fp)) {
  		for(j=0; j<immagine->larghezza; j++){
	      		if (fscanf(fp, "%d", &pixel_val) != 1) 
	      			return;
	      		immagine->pixel[i][j] = pixel_val;
    	}
    	i++;
	}
}

// Funzione che apre l'immagine, legge l'header e i dati dei pixel
// fp: puntatore al file
// immagine: puntatore all'immagine
// filename: nome del file (senza estensione)
// Ritorna 0 se OK, -1 altrimenti
int apri_immagine(FILE *fp, struct image *immagine){

	fp = fopen("feep.pbm", "r");

	if(fp == NULL){
		printf("Errore: Impossibile aprire l'immagine\n");
		return -1;
	}

	
	// Leggo l'header del file PBM
	if(leggi_header(fp, immagine) != 0)
		return -1;
	printf("Info: tipo=%s, larghezza=%d, altezza=%d\n", immagine->tipo, immagine->larghezza, immagine->altezza);
	// Leggo i dati dei pixel
	leggi_dati(fp, immagine);

	fclose(fp);

	return 0;
}

// Funzione che scrive l'hedaer del file PBM
// fp: puntatore al file
// immagine: puntatore all'immagine
// filename: nome con estensione del file
void scrivi_header(FILE *fp, struct image *immagine, char* filename){

	// Scrivo il tipo
	fprintf(fp, "P1\n");

	// Scrivo il commento con il nome del file
	fprintf(fp, "# %s\n", filename);

	// Scrivo le dimensioni
	fprintf(fp, "%d %d\n", immagine->larghezza, immagine->altezza);
}

// Funzione che scrive i dati dei pixel del file PBM
// fp: puntatore al file
// immagine: puntatore all'immagine
void scrivi_dati(FILE *fp, struct image *immagine){
	int i, j;

	for(i=0; i<immagine->altezza; i++){
		for(j=0; j<immagine->larghezza; j++){
			fprintf(fp, "%d",immagine->pixel[i][j]);
		}
		fprintf(fp, "\n");
	}
}

// Funzione che scrive l'header, i dati dei pixel e salva il file PBM
// fp: puntatore al file
// immagine: puntatore all'immagine
// filename: nome del file (senza estensione)
// Ritorna 0 se OK, -1 altrimenti
int salva_immagine(FILE *fp, struct image *immagine, char* filename){

	fp = fopen(filename, "w");

	if(fp == NULL){
		printf("Errore: Impossibile aprire l'immagine\n");
		return -1;
	}

	// Scrivo l'header del file PBM
	scrivi_header(fp, immagine, filename);

	// Scrivo i dati dei pixel
	scrivi_dati(fp, immagine);

	fclose(fp);

	return 0;
}

// Funzione che stampa a video i pixel dell'immagine
// immagine: puntatore all'immagine
void stampa_immagine(struct image *immagine){
	int i,j;

	for(i=0; i<immagine->altezza; i++){
		for(j=0; j<immagine->larghezza; j++){
			printf("%d", immagine->pixel[i][j]);
		}
		printf("\n");
	}
}

// Funzione che ruota di 90 gradi a destra l'immagine
// immagine: puntatore all'immagine originale
// immagine_edited: puntatore all'immagine modificata
void ruota_immagine_90(struct image *immagine, struct image *immagine_edited){
	int i, j;

	// Inverto le dimensioni
	immagine_edited->larghezza = immagine->altezza;
	immagine_edited->altezza = immagine->larghezza;

	printf("Ruoto l'immagine...\n");
	for(i=0; i<immagine->altezza; i++){
		for(j=0; j<immagine->larghezza; j++){
			immagine_edited->pixel[j][immagine_edited->larghezza-1-i] = immagine->pixel[i][j];
		}
	}
}

// Funzione che calcola il negativo dell'immagine
// immagine: puntatore all'immagine originale
// immagine_edited: puntatore all'immagine modificata
void negativo_immagine(struct image *immagine, struct image *immagine_edited){
	int i, j;

	printf("Creo il negativo...\n");
	for(i=0; i<immagine->altezza; i++){
		for(j=0; j<immagine->larghezza; j++){
			immagine_edited->pixel[i][j] = !immagine->pixel[i][j];
		}
	}
}

// Funzione che copia l'immagine
// dest: puntatore all'immagine copia
// source: puntatore all'immagine sorgente
void copia_immagine(struct image *dest, struct image *source){
	int i, j;

	strcpy(dest->tipo, source->tipo);
	dest->larghezza = source->larghezza;
	dest->altezza = source->altezza;
	for(i=0; i<source->altezza; i++){
		for(j=0; j<source->larghezza; j++){
			dest->pixel[i][j] = source->pixel[i][j];
		}
	}
}

int main(int argc, char* argv[]){
	// Variabili immagine
	struct image immagine, immagine_ruotata, immagine_negativa;
	FILE *fp;

	// 1) Apro l'immagine
	apri_immagine(fp, &immagine);

	// 2) Verifico l'immagine caricata
	stampa_immagine(&immagine);

	// 3) Salvo l'immagine in un nuovo file
	salva_immagine(fp, &immagine, "feep_edited.pbm");

	// 4) Ruoto l'immagine
	copia_immagine(&immagine_ruotata, &immagine);
	ruota_immagine_90(&immagine, &immagine_ruotata);
	// 5) Salvo l'immagine ruotata
	salva_immagine(fp, &immagine_ruotata, "feep_rotated.pbm");
	

	// 6) Creo il negativo dell'immagine
	copia_immagine(&immagine_negativa, &immagine);
	negativo_immagine(&immagine, &immagine_negativa);
	//stampa_immagine(&immagine_negativa);
	// 7) Salvo l'immagine negativa
	salva_immagine(fp, &immagine_negativa, "feep_inverted.pbm");

	return 0;
}


